# simulator.py
#
import zg, time, sarg
from graphics import *

win   = GraphWin("circuit",300,300)

class Text :
    def __init__(self, center, text, size=12, color="black") :
        self.center=center; self.text=text; self.size=size; self.color=color

    def draw(self) :
        zg.drawText(win, self.center, self.text, self.size, self.color)

class Wire :
    def __init__(self, name, pts, fedBy, connects=()) :
        self.name=name; self.pts=pts; self.connects=connects
        fedBy.wire = self; self.output=None
        
    def __str__ (self) :
        return "<%s %s>" % (self.name, self.output)

    def takeValue(self, value) :
        if value != self.output :
            self.output = value
            self.recolor()
            for gate,con in self.connects :
                gate.takeInput(con,value)
                queueItem(gate)

    def draw(self) :
        if self.output == None : color = "yellow"
        else :                   color = ["blue","red"][self.output]
        self.lines = zg.drawLines(win,color,False,self.pts,width=2) 

    def recolor(self) :
        if self.output == None : color = "yellow"
        else :                   color = ["blue","red"][self.output]
        for line in self.lines : zg.recolor(line, color)

class Inv :
    def __init__(s, name, center, size) :
        s.name=name; s.center=center; s.size=size; s.output=s.input=None

    def __str__ (s) :
        return "<%s %s %s %s>" % (s.name, s.size, s.input, s.output)

    def takeInput(s, ix, value) :
        s.input = value
        
    def activate (s) :
        if s.input != None : sb = 1-s.input
        if sb != s.output :
            s.output = sb
            s.wire.takeValue(s.output)

    def draw(s) :
        x,y = s.center; size = s.size
        pts = ((x+size,y),(x-size,y-size),(x-size,y+size))
        zg.drawPolygon(win, "yellow", pts, 0)
        sz = size/5.5  
        zg.drawCircle(win, "yellow", (x+size+sz*.9,y), sz, 0) 
    
class Nand :
    def __init__(self, name, center, size) :
        self.name=name; self.center=center; self.size=size
        self.output=None;  self.inputs=[None,None]

    def __str__ (x) :
        return "<%s %s %s %s>" % (x.name, x.size, x.inputs, x.output)

    def takeInput(self, ix, value) :
        self.inputs[ix] = value

    def activate(self) :
        sb = int(not all(self.inputs))
        if sb != self.output :
            self.output = sb
            self.wire.takeValue(sb)

    def draw(self) :
        x,y = self.center; rad=self.size
        zg.drawCircle(win, "yellow", self.center, rad, 0)
        pts = ((x,y+rad),(x-rad,y+rad),(x-rad,y-rad),(x,y-rad))
        zg.drawPolygon(win, "yellow", pts, 0)
        srad = rad/5.5  
        zg.drawCircle(win, "yellow", (x+rad+srad*.9,y), srad, 0) 

class Swt :
    def __init__(s, name, center, size, init=0) :
        s.name=name; s.center=center; s.size=size; s.output=init

    def __str__ (s) :
        return "<%s size=%s output=%s>" % (s.name, s.size, s.output)

    def draw(s) :
        color = ["blue","red"][s.output],
        zg.drawCircle(win, color, s.center, s.size,0)

    def activate(s) :
        s.draw()
        s.wire.takeValue(s.output) # pass signal to wire

    def takeClick(s, pos) :
        xdist = s.center[0]-pos.x
        ydist = s.center[1]-pos.y
        if xdist*xdist + ydist*ydist <= s.size**2 :
            s.output = 1-s.output
            s.activate()

pending = []
def queueItem(item) :
    if item not in pending : pending.append(item)

def gameLoop(gates,trigger=[]) :
    global pending
    pending += trigger
    for item in gates : item.draw()
    clock = 0
    delay = sarg.Float("delay", .01)
    debug = sarg.Int("debug", 0)
    while True:
        if debug and pending :
            print("Pending", [x.name for x in pending])
        for i in range(len(pending)) :
            gate = pending.pop(0)
            if debug : print("gameLoop activating", gate.name)
            gate.activate()
        time.sleep(delay)
        clock += 1

        try    : pos = win.checkMouse()
        except : break   # exit clean if window closed
        if pos : 
            for gate in gates :
                if 'takeClick' in dir(gate) :
                    gate.takeClick(pos)
